/* $Id: common_palette.hpp 54826 2012-07-22 17:22:40Z anonymissimus $ */
/*
   Copyright (C) 2003 - 2012 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/

#ifndef COMMON_PALETTES_H_INCLUDED
#define COMMON_PALETTES_H_INCLUDED

#include "config.hpp"

struct SDL_Rect;

namespace editor {

/**
 * Stores the info about the groups in a nice format.
 */
struct item_group
{
	item_group(const config& cfg):
		id(cfg["id"]), name(cfg["name"].t_str()),
		icon(cfg["icon"]), core(cfg["core"].to_bool()) {};

	std::string id;
	t_string name;
	std::string icon;
    bool core;
};


class common_palette {

public:

	virtual ~common_palette() {}

	//event handling
	virtual bool left_mouse_click(const int x, const int y) = 0;
	virtual bool right_mouse_click(const int x, const int y) = 0;

	/** Scroll the editor-palette up one step if possible. */
	virtual bool scroll_up() = 0;

	/** Scroll the editor-palette down one step if possible. */
	virtual bool scroll_down() = 0;

	//drawing
	virtual void adjust_size(const SDL_Rect& target) = 0;
	virtual void draw(bool) = 0;

	//group
	virtual void set_group(size_t index) = 0;
	virtual void next_group() = 0;
	virtual void prev_group() = 0;
	virtual const config active_group_report() = 0;
	virtual const std::vector<item_group>& get_groups() const = 0;

	/** Menu expanding for palette group list */
	virtual void expand_palette_groups_menu(std::vector<std::string>& items) = 0;

    //item
	virtual size_t num_items() = 0;
	virtual size_t start_num() = 0;
	virtual void set_start_item(size_t index) = 0;
	virtual void swap() = 0;
};


}

#endif
